/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC measure parameters file
 *               
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>             
 *              Martin CECH <256357@mail.muni.cz>
 *               
 *  Date      : $Id: parameters.c 332 2014-03-13 15:12:37Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "configFile.h"
#include "parameters.h"

void MeasureParamsInit(tMeasureParams *p)
{
	p->standalone = 0;
	strcpy(p->standaloneEthName, "eth0");
	p->storeOnlyOneValue = 0;
	p->pulse_mark_by_number = 0;
	p->output = OUTPUT_FILE;

	p->ADC1_on = 0;
	p->ADC1_mode=ADC_CH1_2;
	p->ADC2_on = 0;
	p->ADC2_mode=ADC_CH1_2;

	p->ADC1_CH1_scale = 800;
	p->ADC1_CH2_scale = 800;
	p->ADC2_CH1_scale = 800;
	p->ADC2_CH2_scale = 800;

	p->ADC1_threshold = 0;
	p->ADC2_threshold = 0;

	p->ADC1_offsetPrev = 0;
	p->ADC2_offsetPrev = 0;

	p->ADC1_offsetPost = 0;
	p->ADC2_offsetPost = 0;

	p->ADC1_CH1_offset = 0;
	p->ADC1_CH2_offset = 0;
	p->ADC2_CH1_offset = 0;
	p->ADC2_CH2_offset = 0;

	p->ADC1_integration_delta = 0;
	p->ADC2_integration_delta = 0;
	p->ADC1_integration_sub = 0;
	p->ADC2_integration_sub = 0;

	p->ADC1_fuser_max = 3000;
	p->ADC1_fuser_mult = 3.4255;
	p->ADC1_fuser_add = -608;
	p->ADC2_fuser_max = 3000;
	p->ADC2_fuser_mult = 3.4255;
	p->ADC2_fuser_add = -608;
	
	p->autoStoreSeconds = 0;
	p->autoStopPulses = 0;
}


unsigned short roundUp(unsigned short base, unsigned short input)
{
	return (input + base - 1) / base;
}


int ReadConfigFile(char* fileConfigName, tMeasureParams *p)
{
	//open config file
	FILE *fileConfig = fopen(fileConfigName, "r");
	if (fileConfig == NULL)
	{
		fprintf(stderr, "Can not open configuration file\n");
		return 0;
	}

	t_DATline line;
	int numLine = 0;
	while (1)
	{
		numLine++;
		if (!ReadDATline(fileConfig, &line))
		{
			fprintf(stderr, "Error reading configuration file, line: %d\n", numLine);
			return 0;
		}

		if (line.type == EOF)
		{
			break;
		}
		else if (line.type == LINE_EMPTY)
		{ }
		else if (line.type == LINE_NUMBER)
		{ 
			fprintf(stderr, "unsupported line type, line: %d\n", numLine);
			return 0;	
		}
		else if (line.type == LINE_PARAMETER)
		{ 
			if (strcasecmp(line.paramName, "ADC1") == 0)
			{
				if (strcasecmp(line.paramValue, "on") == 0)
				{
					p->ADC1_on = 1;
					printf("ADC1:                    on\n");	
				}
				else if (strcasecmp(line.paramValue, "off") == 0)
				{
					p->ADC1_on = 0;
					printf("ADC1:                    off\n"); 
				}
				else
				{
					fprintf(stderr, "ADC1 - unknown value, line: %d\n", numLine);
					return 0;
				} 
			}
			else if (strcasecmp(line.paramName, "ADC2") == 0)
			{
				if (strcasecmp(line.paramValue, "on") == 0)
				{
					p->ADC2_on = 1;
					printf("ADC2:                    on\n");	
				}
				else if (strcasecmp(line.paramValue, "off") == 0)
				{
					p->ADC2_on = 0;
					printf("ADC2:                    off\n"); 
				}
				else
				{
					fprintf(stderr, "ADC2 - unknown value, line: %d\n", numLine);
					return 0;
				} 
			}
			else if (strcasecmp(line.paramName, "STORE_ONLY_ONE_VALUE") == 0)
			{
				if (strcasecmp(line.paramValue, "on") == 0)
				{
					p->storeOnlyOneValue = 1;
					printf("Store one value:         on\n");	
				}
			}
			else if (strcasecmp(line.paramName, "ADC1&2_MODE") == 0){
				if (strcasecmp(line.paramValue, "test_pattern_mode") == 0)
				{
					p->ADC1_mode = ADC_tpm;
					p->ADC2_mode = ADC_tpm;
					printf("ADC1&2:                  set test pattern mode on both channels\n");
				}
				else if (strcasecmp(line.paramValue, "time_test_mode") == 0)
				{
					p->ADC1_mode = ADC_ttm;
					p->ADC2_mode = ADC_ttm;
					printf("ADC1&2:                  set time test mode on both channels\n");
				}
				else
				{
					fprintf(stderr, "ADC1&2_MODE - unknown mode %s, line: %d\n", line.paramValue, numLine);
					return 0;
				}


			}

			else if (strcasecmp(line.paramName, "ADC1_MODE") == 0)
			{
				if ((p->ADC1_mode == ADC_tpm) || (p->ADC1_mode == ADC_ttm)) continue;
				if (strcasecmp(line.paramValue, "CH1_slow") == 0)
				{
					p->ADC1_mode = ADC_CH1;
					printf("ADC1_mode:               set channel 1, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1_fast") == 0)
				{
					p->ADC1_mode = ADC_CH1_fast;
					printf("ADC1_mode:               set channel 1, fast speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH2_fast") == 0)
				{
					p->ADC1_mode = ADC_CH2_fast;
					printf("ADC1_mode:               set channel 2, fast speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH2_slow") == 0)
				{
					p->ADC1_mode = ADC_CH2;
					printf("ADC1_mode:               set channel 2, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1&2_slow") == 0)
				{
					p->ADC1_mode = ADC_CH1_2;
					printf("ADC1_mode:               set channel 1 and 2, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1&2_fused") == 0)
				{
					p->ADC1_mode = ADC_fused;
					printf("ADC1_mode:               fused channels 1 and 2\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_CH1") == 0)
				{
					p->ADC1_mode = ADC_INTM_CH1;
					printf("ADC1_mode:               integration method on channel 1\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_CH2") == 0)
				{
					p->ADC1_mode = ADC_INTM_CH2;
					printf("ADC1_mode:               integration method on channel 2\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_fused") == 0)
				{
					p->ADC1_mode = ADC_INTM_fused;
					printf("ADC1_mode:               integration method on fused channels 1 and 2\n");
				}
				else
				{
					fprintf(stderr, "ADC1_MODE - unknown mode %s, line: %d\n", line.paramValue, numLine);
					return 0;
				}
			}


			else if (strcasecmp(line.paramName, "ADC2_MODE") == 0)
			{
				if ((p->ADC2_mode == ADC_tpm) || (p->ADC2_mode == ADC_ttm)) continue;
				if (strcasecmp(line.paramValue, "CH1_slow") == 0)
				{
					p->ADC2_mode = ADC_CH1;
					printf("ADC2_mode:               set channel 1, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1_fast") == 0)
				{
					p->ADC2_mode = ADC_CH1_fast;
					printf("ADC2_mode:               set channel 1, fast speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH2_fast") == 0)
				{
					p->ADC2_mode = ADC_CH2_fast;
					printf("ADC2_mode:               set channel 2, fast speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH2_slow") == 0)
				{
					p->ADC2_mode = ADC_CH2;
					printf("ADC2_mode:               set channel 2, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1&2_slow") == 0)
				{
					p->ADC2_mode = ADC_CH1_2;
					printf("ADC2_mode:               set channel 1 and 2, low speed\n");
				}
				else if (strcasecmp(line.paramValue, "CH1&2_fused") == 0)
				{
					p->ADC2_mode = ADC_fused;
					printf("ADC2_mode:               fused channels 1 and 2\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_CH1") == 0)
				{
					p->ADC2_mode = ADC_INTM_CH1;
					printf("ADC2_mode:               integration method on channel 1\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_CH2") == 0)
				{
					p->ADC2_mode = ADC_INTM_CH2;
					printf("ADC2_mode:               integration method on channel 2\n");
				}
				else if (strcasecmp(line.paramValue, "INTEGRATION_fused") == 0)
				{
					p->ADC2_mode = ADC_INTM_fused;
					printf("ADC2_mode:               integration method on fused channels 1 and 2\n");
				}
				else
				{
					fprintf(stderr, "ADC2_MODE - unknown mode %s, line: %d\n", line.paramValue, numLine);
					return 0;
				}
			}


			
			else if (strcasecmp(line.paramName, "PULSE_MARK_BY_NUMBER") == 0)
			{
				if (strcasecmp(line.paramValue, "yes") == 0)
				{
					p->pulse_mark_by_number = 1;
					printf("Pulse mark by number:    yes\n");
				}
				else if (strcasecmp(line.paramValue, "no") == 0)
				{
					p->pulse_mark_by_number = 0;
					printf("Pulse mark by number:    no\n");
				}
				else
				{
					fprintf(stderr, "PULSE_MARK_BY_NUMBER - unknown value, line: %d\n", numLine);
					return 0;
				}		 
			}
			
			
			
			else if (strcasecmp(line.paramName, "UDP_OUTPUT") == 0)
			{
				p->output = OUTPUT_UDP;
				memcpy(p->UDP_IP_address, line.paramValue, 25);
				printf("Method output send to:   %s\n", p->UDP_IP_address);
			}



			else if (strcasecmp(line.paramName, "ADC1_THRESHOLD_VALUE") == 0)
			{
				p->ADC1_threshold &= 0xF000;	//clear old value
				unsigned short threshold = ((unsigned short)atoi(line.paramValue)) & 0x0FFF;	//only low 12 bits is used
				p->ADC1_threshold |= threshold;
				printf("ADC1 threshold:          %u\n", threshold);		 
			}
			else if (strcasecmp(line.paramName, "ADC1_THRESHOLD_CHANNEL") == 0)
			{
				if (strcasecmp(line.paramValue, "CH1") == 0)
				{
					p->ADC1_threshold &= 0x0FFF;	//clear old value
					p->ADC1_threshold |= 0x1000;
					p->ADC1_threshold_channel = "CH1";
					printf("ADC1 threshold channel:  CH1\n");
				}
				else if (strcasecmp(line.paramValue, "CH2") == 0)
				{
					p->ADC1_threshold &= 0x0FFF;	//clear old value
					p->ADC1_threshold |= 0x2000;
					p->ADC1_threshold_channel = "CH2";
					printf("ADC1 threshold channel:  CH2\n");
				}
				else
				{
					fprintf(stderr, "ADC1_THRESHOLD_CHANNEL - unknown value, line: %d\n", numLine);
					return 0;
				}
			}



			else if (strcasecmp(line.paramName, "ADC2_THRESHOLD_VALUE") == 0)
			{
				p->ADC2_threshold &= 0xF000;	//clear old value
				unsigned short threshold = ((unsigned short)atoi(line.paramValue)) & 0x0FFF;	//only low 12 bits is used
				p->ADC2_threshold |= threshold;
				printf("ADC2 threshold:          %u\n", threshold);		 
			}
			else if (strcasecmp(line.paramName, "ADC2_THRESHOLD_CHANNEL") == 0)
			{
				if (strcasecmp(line.paramValue, "CH1") == 0)
				{
					p->ADC2_threshold &= 0x0FFF;	//clear old value
					p->ADC2_threshold |= 0x1000;
					p->ADC2_threshold_channel = "CH1";
					printf("ADC2 threshold channel:  CH1\n");
				}
				else if (strcasecmp(line.paramValue, "CH2") == 0)
				{
					p->ADC2_threshold &= 0x0FFF;	//clear old value
					p->ADC2_threshold |= 0x2000;
					p->ADC2_threshold_channel = "CH2";
					printf("ADC2 threshold channel:  CH2\n");
				}
				else
				{
					fprintf(stderr, "ADC2_THRESHOLD_CHANNEL - unknown value, line: %d\n", numLine);
					return 0;
				}
			}



			else if (strcasecmp(line.paramName, "ADC1_THRESHOLD_OFFSET_PREV") == 0)
			{
				p->ADC1_offsetPrev = roundUp(HW_SAMPLES_IN_1_PERIOD, ((unsigned short)atoi(line.paramValue)) & 0x00FF);	//only low 8 bits is used
				printf("ADC1 offset prev:        %u\n", p->ADC1_offsetPrev * HW_SAMPLES_IN_1_PERIOD);		 
			}
			else if (strcasecmp(line.paramName, "ADC2_THRESHOLD_OFFSET_PREV") == 0)
			{
				p->ADC2_offsetPrev = roundUp(HW_SAMPLES_IN_1_PERIOD, ((unsigned short)atoi(line.paramValue)) & 0x00FF);	//only low 8 bits is used
				printf("ADC2 offset prev:        %u\n", p->ADC2_offsetPrev * HW_SAMPLES_IN_1_PERIOD);		 
			}



			else if (strcasecmp(line.paramName, "ADC1_THRESHOLD_OFFSET_POST") == 0)
			{
				p->ADC1_offsetPost = roundUp(HW_SAMPLES_IN_1_PERIOD, (unsigned short)atoi(line.paramValue));
				printf("ADC1 offset post:        %u\n", p->ADC1_offsetPost * HW_SAMPLES_IN_1_PERIOD);
			}
			else if (strcasecmp(line.paramName, "ADC2_THRESHOLD_OFFSET_POST") == 0)
			{
				p->ADC2_offsetPost = roundUp(HW_SAMPLES_IN_1_PERIOD, (unsigned short)atoi(line.paramValue));
				printf("ADC2 offset post:        %u\n", p->ADC2_offsetPost * HW_SAMPLES_IN_1_PERIOD);
			}



			else if (strcasecmp(line.paramName, "ADC1_CH1_SCALE_mV") == 0)
			{
				p->ADC1_CH1_scale = (float)atof(line.paramValue);
				printf("ADC1 CH1 scale:          %1.1fmV\n", p->ADC1_CH1_scale);
			}
			else if (strcasecmp(line.paramName, "ADC1_CH2_SCALE_mV") == 0)
			{
				p->ADC1_CH2_scale = (float)atof(line.paramValue);
				printf("ADC1 CH2 scale:          %1.1fmV\n", p->ADC1_CH2_scale);
			}
			else if (strcasecmp(line.paramName, "ADC2_CH1_SCALE_mV") == 0)
			{
				p->ADC2_CH1_scale = (float)atof(line.paramValue);
				printf("ADC2 CH1 scale:          %1.1fmV\n", p->ADC2_CH1_scale);
			}
			else if (strcasecmp(line.paramName, "ADC2_CH2_SCALE_mV") == 0)
			{
				p->ADC2_CH2_scale = (float)atof(line.paramValue);
				printf("ADC2 CH2 scale:          %1.1fmV\n", p->ADC2_CH2_scale);
			}



			else if (strcasecmp(line.paramName, "ADC1_CH1_OFFSET_mV") == 0)
			{
				p->ADC1_CH1_offset = (float)atof(line.paramValue);
				printf("ADC1 CH1 offset:         %1.1fmV\n", p->ADC1_CH1_offset);
			}
			else if (strcasecmp(line.paramName, "ADC1_CH2_OFFSET_mV") == 0)
			{
				p->ADC1_CH2_offset = (float)atof(line.paramValue);
				printf("ADC1 CH2 offset:         %1.1fmV\n", p->ADC1_CH2_offset);
			}
			else if (strcasecmp(line.paramName, "ADC2_CH1_OFFSET_mV") == 0)
			{
				p->ADC2_CH1_offset = (float)atof(line.paramValue);
				printf("ADC2 CH1 offset:         %1.1fmV\n", p->ADC2_CH1_offset);
			}
			else if (strcasecmp(line.paramName, "ADC2_CH2_OFFSET_mV") == 0)
			{
				p->ADC2_CH2_offset = (float)atof(line.paramValue);
				printf("ADC2 CH2 offset:         %1.1fmV\n", p->ADC2_CH2_offset);
			}

			else if (strcasecmp(line.paramName, "STANDALONE") == 0)
			{
				strcpy(p->standaloneEthName, line.paramValue);
				p->standalone |= 1;
				printf("STANDALONE:              TRUE (%s)\n", p->standaloneEthName);
			}

			else if (strcasecmp(line.paramName, "STANDALONE_OUTPUT") == 0)
			{
				if (strcasecmp(line.paramValue, "ADC1") == 0)
				{
					p->standalone |= 2;
				    printf("STANDALONE output:       ADC1\n");
				}
				else if (strcasecmp(line.paramValue, "ADC2") == 0)
				{
					p->standalone |= 4;
				    printf("STANDALONE output:       ADC2\n");
				}
				else
				{
					fprintf(stderr, "STANDALONE_OUTPUT - unknown value, line: %d\n", numLine);
					return 0;
				}
			}

			else if (strcasecmp(line.paramName, "ADC1_CH_FUSER_mult") == 0)
			{
				p->ADC1_fuser_mult = (float)atof(line.paramValue);
				printf("ADC1 FUSER multiplicative constant:  %1.5f\n", p->ADC1_fuser_mult);
			}

			else if (strcasecmp(line.paramName, "ADC1_CH_FUSER_max") == 0)
			{
				p->ADC1_fuser_max =  ((unsigned short)atoi(line.paramValue)) & 0x0FFF; //only 12 bit value
				printf("ADC1 FUSER CH2 maximum:              %u\n", p->ADC1_fuser_max);
			}

			else if (strcasecmp(line.paramName, "ADC1_CH_FUSER_add") == 0)
			{
				p->ADC1_fuser_add =  ((unsigned short)atoi(line.paramValue)) & 0xFFFF; //only 16 bit value
				printf("ADC1 FUSER aditive constant:         %d\n", atoi(line.paramValue));
			}

			else if (strcasecmp(line.paramName, "ADC2_CH_FUSER_mult") == 0)
			{
				p->ADC2_fuser_mult = (float)atof(line.paramValue);
				printf("ADC2 FUSER multiplicative constant:  %1.5f\n", p->ADC2_fuser_mult);
			}

			else if (strcasecmp(line.paramName, "ADC2_CH_FUSER_max") == 0)
			{
				p->ADC2_fuser_max =  ((unsigned short)atoi(line.paramValue)) & 0x0FFF; //only 12 bit value
				printf("ADC2 FUSER CH2 maximum:              %u\n", p->ADC2_fuser_max);
			}

			else if (strcasecmp(line.paramName, "ADC2_CH_FUSER_add") == 0)
			{
				p->ADC2_fuser_add =  ((unsigned short)atoi(line.paramValue)) & 0xFFFF; //only 16 bit value
				printf("ADC2 FUSER aditive constant:         %d\n", atoi(line.paramValue));
			}

			else if (strcasecmp(line.paramName, "ADC1_METHOD_INTEGRATION_delta") == 0)
			{
				p->ADC1_integration_delta =  ((unsigned short)atoi(line.paramValue)) & 0x00FF; //only 8 bit value
				printf("ADC1 integration method delta:       %u\n", p->ADC1_integration_delta);
			}
			else if (strcasecmp(line.paramName, "ADC2_METHOD_INTEGRATION_delta") == 0)
			{
				p->ADC2_integration_delta =  ((unsigned short)atoi(line.paramValue)) & 0x00FF; //only 8 bit value
				printf("ADC2 integration method delta:       %u\n", p->ADC2_integration_delta);
			}

			else if (strcasecmp(line.paramName, "ADC1_METHOD_INTEGRATION_sub") == 0)
			{
				p->ADC1_integration_sub =  ((unsigned short)atoi(line.paramValue)) & 0xFFFF; //only 16 bit value
				printf("ADC1 integration method sub:         %u\n", p->ADC1_integration_sub);
			}
			else if (strcasecmp(line.paramName, "ADC2_METHOD_INTEGRATION_sub") == 0)
			{
				p->ADC2_integration_sub =  ((unsigned short)atoi(line.paramValue)) & 0xFFFF; //only 16 bit value
				printf("ADC2 integration method sub:         %u\n", p->ADC2_integration_sub);
			}
			
			else if (strcasecmp(line.paramName, "AUTO_STORE_[seconds]") == 0)
			{
				p->autoStoreSeconds |= ((unsigned int)atoi(line.paramValue));
				printf("AUTO STORE [seconds]:          %u\n", p->autoStoreSeconds);		 
			}
			else if (strcasecmp(line.paramName, "AUTO_STOP_[pulses]") == 0)
			{
				p->autoStopPulses |= ((unsigned int)atoi(line.paramValue));
				printf("AUTO STOP [pulses]:          %u\n", p->autoStopPulses);		 
			}

			else
			{
				fprintf(stderr, "unknown parameter, line: %d\n", numLine);
				return 0;
			}				 
		}
		else
		{
			fprintf(stderr, "Error reading configuration file - undefined line, line: %d\n", numLine);
			return 0;
		}
	} 

	fclose(fileConfig);

	p->standalone &= 0xFF * (p->standalone & 1);
	//if dual edge samling mode is enable (measure at fast speed on one channel) -> threshold have to be enabled on both channels
	if (p->ADC1_mode < ADC_CH2_fast){
		p->ADC1_threshold &= 0x0FFF;
	}
	if (p->ADC2_mode < ADC_CH2_fast){
		p->ADC2_threshold &= 0x0FFF;
	}
	if ((p->ADC1_mode>=ADC_fused) && (p->ADC1_mode<=ADC_INTM_fused)){
		p->ADC1_threshold &= 0x0FFF;
	}
	if ((p->ADC2_mode>=ADC_fused) && (p->ADC2_mode<=ADC_INTM_fused)){
		p->ADC2_threshold &= 0x0FFF;
	}

	p->ACD12_dualEdgeSamplingMode = 0;
	p->ACD12_dualEdgeSamplingMode_channel=0;
	p->ACD12_testPatternMode=0;
	p->ACD12_timeTestMode=0;
	p->storeChannel1=0;
	p->storeChannel2=0;

	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! rozhodnout na zklad aktivnho ADC
	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	switch(p->ADC2_mode){
	case ADC_CH1_fast:
		p->ACD12_dualEdgeSamplingMode = 1;
		p->ACD12_dualEdgeSamplingMode_channel=0;
		p->storeChannel1=1;
		p->storeChannel2=0;
		break;
	case ADC_CH2_fast:
		p->ACD12_dualEdgeSamplingMode = 1;
		p->ACD12_dualEdgeSamplingMode_channel=1;
		p->storeChannel1=0;
		p->storeChannel2=1;
		break;
	case ADC_CH1:
		p->storeChannel1=1;
		p->storeChannel2=0;
		break;
	case ADC_CH2:
		p->storeChannel1=0;
		p->storeChannel2=1;
		break;
	case ADC_CH1_2:
		p->storeChannel1=1;
		p->storeChannel2=1;
		break;
	case ADC_ttm:
		p->ACD12_timeTestMode=1;
		break;
	case ADC_tpm:
		p->ACD12_testPatternMode=1;
		break;
	case ADC_INTM_CH1:
		p->storeChannel1=1;
		break;
	case ADC_INTM_CH2:
		p->storeChannel2=1;
		break;
	case ADC_INTM_fused: case ADC_fused:
		p->storeChannel1=1;
		break;
	default:break;
	}
	return 1; 
}

unsigned short transformScale(float input, float minFloat, float maxFloat, unsigned short scale)
{
	input = input < minFloat ? minFloat : input;
	input = input > maxFloat ? maxFloat : input;

	return (unsigned short)(((input - minFloat) / (maxFloat - minFloat)) * (float)scale + 0.5);
}

unsigned short transformScaleCHOffset(float input){transformScale(input, 0.0, 45.0, 0x0FFF);}
unsigned short transformScaleCHRange(float input){transformScale(input, 600.0, 1000.0, 0x7FFF);}

inline unsigned short transformFuserMult(float input)
{
	return (unsigned short) ((input*(1<<13))+0.5);
}
