/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC measure parameters file header file
 *               
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>             
 *              Martin CECH <256357@mail.muni.cz>
 *               
 *  Date      : $Id: parameters.h 332 2014-03-13 15:12:37Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */
 
#ifndef __PARAMETERS_H_
#define __PARAMETERS_H_

#define OFFSET_PREV 1
#define OFFSET_POST 1

#define HW_SAMPLES_IN_1_PERIOD 16

typedef enum eMeasureModes
{
	ADC_CH1_fast=0,
	ADC_CH2_fast,
	ADC_tpm,
	ADC_ttm=3,
	ADC_fused=4,
	ADC_INTM_fused=5,
	ADC_CH1=6,
	ADC_CH2,
	ADC_CH1_2,
	ADC_INTM_CH1,
	ADC_INTM_CH2
}tMeasureModes;

typedef enum eOutputModes
{
	OUTPUT_FILE,
	OUTPUT_UDP
}tOutputModes;

/**
 * Data type for storing all the parameters about measurement.
 * Its values are loaded from configuration file
 */
typedef struct sMeasureParams
{
	unsigned char standalone;
	unsigned char standaloneEthName[200];

	unsigned char storeOnlyOneValue;

	unsigned char ADC1_on;
	tMeasureModes ADC1_mode;
	unsigned short ADC1_threshold;//contain threshold value on 12 low bits and threshold type (CH1 or CH2 or CH1&2 or edge detector) on 4 high bits
	char *ADC1_threshold_channel;
	unsigned short ADC1_offsetPrev;
	unsigned short ADC1_offsetPost;
	float ADC1_CH1_scale;
	float ADC1_CH2_scale;
	float ADC1_CH1_offset;
	float ADC1_CH2_offset;
	unsigned char ADC1_integration_delta;
	unsigned short ADC1_integration_sub;
	unsigned short ADC1_fuser_max;
	float ADC1_fuser_mult;
	unsigned short ADC1_fuser_add;

	unsigned char ADC2_on;
	tMeasureModes ADC2_mode;
	unsigned short ADC2_threshold;
	char *ADC2_threshold_channel;
	unsigned short ADC2_offsetPrev;
	unsigned short ADC2_offsetPost;
	float ADC2_CH1_scale;
	float ADC2_CH2_scale;
	float ADC2_CH1_offset;
	float ADC2_CH2_offset;
	unsigned char ADC2_integration_delta;
	unsigned short ADC2_integration_sub;
	unsigned short ADC2_fuser_max;
	float ADC2_fuser_mult;
	unsigned short ADC2_fuser_add;

	unsigned char pulse_mark_by_number;		//otherwise the time is used
	unsigned char output;			//UDP or file
	char          UDP_IP_address[50];

	unsigned char ADC12_oen;
	unsigned char ADC12_calib;
	
	unsigned int autoStoreSeconds;	//save measuring after expected seconds and calibrate ADCs
	unsigned int autoStopPulses;	//stop measuring after canching expected number of pulses

	//TODO odstranit a nahradit testy mdu
	unsigned char ACD12_dualEdgeSamplingMode;
	unsigned char ACD12_dualEdgeSamplingMode_channel;
	unsigned char ACD12_testPatternMode;
	unsigned char ACD12_timeTestMode;
	unsigned char storeChannel1;
	unsigned char storeChannel2;


	unsigned char bitfile[200];
} tMeasureParams;

/**
 * This procedure inits tMeasureParams structure to default values. Should be called immediatelly
 * after creating variable of tMeasureParams
 * @param p variable to initialize
 */
void MeasureParamsInit(tMeasureParams *p);
/**
 * Read configuration file and save all values to tMeasureParams structure.
 * @param fileConfigName name of configuration file (path to this file)
 * @param p structure tMeasureParams for storing parameters
 * @return 1 after successful reading of config file, 0 otherwise and problem is printed to stderr
 */
int ReadConfigFile(char* fileConfigName, tMeasureParams *p);

/**
 * Transforms float value of channel offset to 16 bit value which can be used
 * directly as an input to ADC register (I and Q offset register).
 * @param input is float value of channel offset
 * @return 16 bit value
 */
inline unsigned short transformScaleCHOffset(float input);

/**
 * Transforms float value of channel scale to 16 bit value which can be used
 * directly as an input to ADC register (I nad Q FSR register).
 * @param input is float value of channel
 * @return 16 bit value
 */
inline unsigned short transformScaleCHRange(float input);

/**
 * Transforms float value of channel 1:channel 2 multiplication to 16 bit value which can be used
 * directly as an input to registers.
 * @param input is float value - multiplication
 * @return 16 bit value
 */
inline unsigned short transformFuserMult(float input);
#endif /* PARAMETERS_H_ */
