/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC program for ADC card control and data acquisition
 *
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz> 
 *              Martin CECH <256357@mail.muni.cz> 
 *
 *  Date      : $Id: spektromeasure.c 332 2014-03-13 15:12:37Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>

#include "spektromeasure.h"
#include "storingData.h"
#include "parameters.h"
#include "testing.h"
#include "standalone.h"
#include "UDPsending.h"



int runProgram = 1;
double hwPeriod_ns;
tMeasureParams measureParams;
tBuffer b;
time_t previousTime;
unsigned long long numPackets = 0;

int channelADC1 = 0x1;
int channelADC2 = 0x2;

//catch ctrl+C
void signal_handler(int signum)
{
	runProgram = 0;
}

void PacketDecodingInfoInit(tPacketDecodingInfo *i, char ID)
{
	i->prevPacketTimeEnd_HWticks = 0;
	i->prevDataStamp = 0;
	i->prevPacketStamp = 0;

	i->firstRun = 1;

	i->ID = ID;

	TestParamsInit(&(i->testParams));
	HDDParamsInit(&(i->HDDparams));

	i->catchedPackets = 0;
	i->lostPackets = 0;
	i->catchedPacketsPrevious = 0;
	i->lostPacketsPrevious = 0;
}



int bufferInit(tBuffer* b, unsigned long long bufferLength)
{
	b->bufPointerWrite = b->bufPointerRead = b->buffer = malloc(bufferLength);
	if (b->buffer == NULL)
	{
		fprintf(stderr, "Can not allocate main buffer. Probably is installed less memory then buffer size required.\n");
		return 0;
	}

	b->bufStop = b->buffer + bufferLength - PACKET_MAX_LENGTH;

	b->bufferedPackets[0] = b->bufferedPackets[1] = 0;
	return 1;
}


void PrintSavedToHDD()
{
	printf("%c[1A", 27);	//VT100 escape code
	printf(    "Save to HDD       | %16llu |\n", numPackets);
	fflush(stdout);
}

unsigned int oldSpeed = 0;

void PrintBufferedPackets(int printR, int printNewSpeed, int jumpUp)
{
	unsigned long long buffPckts = b.bufferedPackets[0] + b.bufferedPackets[1];
	unsigned long long buffPcktsPrev = b.previousBufferedPackets[0] + b.previousBufferedPackets[1];

	unsigned speed = buffPckts - buffPcktsPrev;
	if (!printNewSpeed)
	{
		speed = oldSpeed;
	}
	oldSpeed = speed;
	b.previousBufferedPackets[0] = b.bufferedPackets[0];
	b.previousBufferedPackets[1] = b.bufferedPackets[1];
	
	if (jumpUp)
	{
		printf("%c[2A", 27);	//VT100 escape code
	}
	printf("                                                                                \n");
	printf("Buffering packets | %16llu ||    Speed (s-1) | %16llu |\n", buffPckts, speed);
	
	fflush(stdout);
}

unsigned int UDPpacketNumberPrev = 0;

void PrintUDPStatistics(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2, int jumpUp)
{
	time_t actualTime;
	actualTime = time ( NULL );
	if (actualTime != previousTime)
	{
		if (jumpUp)
		{
			printf("%c[4A", 27);	//VT100 escape code
		}
	printf("                                                                                \n");
		printf("Catched packets    %16llu |      Speed (s-1) %10llu |     \n", pdi_ADC1->catchedPackets + pdi_ADC2->catchedPackets, (pdi_ADC1->catchedPackets + pdi_ADC2->catchedPackets) - (pdi_ADC1->catchedPacketsPrevious + pdi_ADC2->catchedPacketsPrevious));
		printf("Send packets       %16llu |      Speed (s-1) %10llu |     \n", UDPpacketNumber, UDPpacketNumber - UDPpacketNumberPrev);
		printf("Lost packets       %16llu |      Speed (s-1) %10llu |     \n", pdi_ADC1->lostPackets + pdi_ADC2->lostPackets, (pdi_ADC1->lostPackets + pdi_ADC2->lostPackets) - (pdi_ADC1->lostPacketsPrevious + pdi_ADC2->lostPacketsPrevious));
		fflush(stdout);
		previousTime = actualTime;
		
		pdi_ADC1->catchedPacketsPrevious = pdi_ADC1->catchedPackets;
		pdi_ADC2->catchedPacketsPrevious = pdi_ADC2->catchedPackets;
		pdi_ADC1->lostPacketsPrevious = pdi_ADC1->lostPackets;
		pdi_ADC2->lostPacketsPrevious = pdi_ADC2->lostPackets;
		
		UDPpacketNumberPrev = UDPpacketNumber;
	}
}

void PrintCheckedPackets(unsigned long long ADC1, unsigned long long ADC2)
{
	printf("%c[2A", 27);	//VT100 escape code
	printf("                                                                                \n");
	printf("Checked packets ADC1: %15llu     Checked packets ADC2: %15llu\n", ADC1, ADC2);
	fflush(stdout);
}

unsigned short GetSampleA(unsigned char *data, int i)
{
	return ((unsigned short)data[i] << 4 ) | ((unsigned short)data[i+1] >> 4 );
}

unsigned short GetSampleB(unsigned char *data, int i)
{
	return (((unsigned short)data[i+1] & 0xF) << 8 ) | (unsigned short)data[i+2];
}

void BufferPacket(tPacketDecodingInfo *pdi)
{
	const unsigned char *packet;
	unsigned int packet_len;
	if (measureParams.standalone){
		packet = readPcapNext(&packet_len);
	}else{
		
	}
	if (packet != NULL)
	{
		*((unsigned int *)b.bufPointerWrite) = packet_len;
		b.bufPointerWrite += sizeof(unsigned int);

		memcpy(b.bufPointerWrite, packet, packet_len);
		b.bufPointerWrite += packet_len;
		b.bufferedPackets[packet[PACKET_HEADER_ETHERNET_LENGTH]]++;
	}

	//write statistics - number of buffered packets
	if (!measureParams.ACD12_testPatternMode && !measureParams.ACD12_timeTestMode && measureParams.output == OUTPUT_FILE)
	{
		//write num packets stored to HDD
		time_t actualTime;
		actualTime = time ( NULL );
		if (actualTime != previousTime)
		{
			PrintBufferedPackets(1, 1, 1);
			previousTime = actualTime;
		}
	}
}

void DecodeAndStorePacket(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2)
{
	unsigned char *data, *hw_data;
	unsigned int data_len, hw_data_len;
	unsigned int segsize;

	short i;
	data_len = *((unsigned int *)b.bufPointerRead);
	b.bufPointerRead += sizeof(unsigned int);

	data = b.bufPointerRead;
	b.bufPointerRead += data_len;
	numPackets++;

	tPacketDecodingInfo *pdi = (data[PACKET_HEADER_ETHERNET_LENGTH] == pdi_ADC1->ID) ? pdi_ADC1 : pdi_ADC2;
	tMeasureModes mode = pdi == pdi_ADC1? measureParams.ADC1_mode:measureParams.ADC2_mode;

	//packet structure: 6+6+2B packet header (address etc.), 4B packet stamp, 8B time stamp, 4B data stamp, 2B reserved, variable length - data, 1B - footer

	//parse time information from packet (convert between little endian and big endian)
	unsigned int packetStamp = 	((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+1] << 24 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+2] << 16 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+3] << 8 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+4]);

	unsigned long long timeStamp_HWticks = 	((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+5] << 56 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+6] << 48 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+7] << 40 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+8] << 32 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+9] << 24 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+10] << 16 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+11] << 8 ) |
			((unsigned long long)data[PACKET_HEADER_ETHERNET_LENGTH+12]);

	unsigned int dataStamp = 	((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+13] << 24 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+14] << 16 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+15] << 8 ) |
			((unsigned int)data[PACKET_HEADER_ETHERNET_LENGTH+16]);

	unsigned char FP_LP = data[data_len - 1];

	//check packet length
	if ((data_len - PACKET_HEADER_LENGTH - PACKET_FOOTER_LENGTH) % 12*HW_SAMPLES_IN_1_PERIOD/8 != 0)
	{
		fprintf(stderr, "error - packet length is not correct. Lenght is %d\n", data_len);
		return;
	}

	//test pattern mode
	if (mode==ADC_tpm)
	{
		char* dataToCheck = data + PACKET_HEADER_LENGTH;
		unsigned dataLenTocheck = data_len - PACKET_HEADER_LENGTH - PACKET_FOOTER_LENGTH;

		CheckTestPattern(dataToCheck, dataLenTocheck, timeStamp_HWticks, &(pdi->testParams));

		if (pdi->testParams.testPatter_checkedPackets % 0x10000 == 0)
		{
			PrintCheckedPackets(pdi_ADC1->testParams.testPatter_checkedPackets, pdi_ADC2->testParams.testPatter_checkedPackets);
		}

		return;
	}

	//time test mode
	if (mode==ADC_ttm)
	{
		char* dataToCheck = data + PACKET_HEADER_LENGTH;
		unsigned dataLenTocheck = data_len - PACKET_HEADER_LENGTH - PACKET_FOOTER_LENGTH;

		TimeTestMode_TestPacket(packetStamp, dataStamp, timeStamp_HWticks, FP_LP, dataToCheck, dataLenTocheck);

		return;
	}

	//save statistics
	pdi->catchedPackets++;
	
	//check for data lost
	unsigned int numLostPackets = 0;
	unsigned long long beginTime = 0;
	unsigned long long endTime = 0;
	if (packetStamp != pdi->prevPacketStamp + 1 && !pdi->firstRun)
	{
		numLostPackets = packetStamp - pdi->prevPacketStamp - 1;
		beginTime = (unsigned long long)(pdi->prevPacketTimeEnd_HWticks * hwPeriod_ns);
		endTime = (unsigned long long)(timeStamp_HWticks * hwPeriod_ns);

		//get statistics
		pdi->lostPackets += numLostPackets;
	}
	
	//write integration method result to file
	if ((mode == ADC_INTM_CH1) || (mode == ADC_INTM_CH2) || (mode == ADC_INTM_fused))
	{
		unsigned long valueA = (data[data_len-3]<<8) | data[data_len-2];
		unsigned long valueB = (data[data_len-7]<<24) | (data[data_len-6]<<16) | (data[data_len-5]<<8) | data[data_len-4];
		
		//save integration method output to file
		if (measureParams.output == OUTPUT_FILE)
		{
			pdi->HDDparams.bufHDDpointer_CH1 += sprintf(pdi->HDDparams.bufHDDpointer_CH1, "%llu", (unsigned long long)(dataStamp));
			WriteDecimal(&(pdi->HDDparams.bufHDDpointer_CH1), valueA);
			WriteDecimal(&(pdi->HDDparams.bufHDDpointer_CH1), valueB);
			pdi->HDDparams.bufHDDpointer_CH1 += sprintf(pdi->HDDparams.bufHDDpointer_CH1, "\n");
		}
		
		//send value via UDP packet to remote device
		else if (measureParams.output == OUTPUT_UDP)
		{
			SendPulseViaUDP(timeStamp_HWticks, dataStamp, valueA, valueB);
		}
		else
		{
			fprintf(stderr, "Unknown type of data storing.\n");
			return;
		}
	}
	
	//write RAW data to file or send via UDP packet
	else
	{
		if (measureParams.output == OUTPUT_UDP)
		{
			SendUDPpacket(data, data_len);
		}
		else if (measureParams.output == OUTPUT_FILE)
		{
			//store packet lost
			if (numLostPackets > 0)
			{
				if (measureParams.storeChannel1)
				{
					WritePacketsLost(&(pdi->HDDparams.bufHDDpointer_CH1), numLostPackets, beginTime, endTime);
				}
				if (measureParams.storeChannel2)
				{
					WritePacketsLost(&(pdi->HDDparams.bufHDDpointer_CH2), numLostPackets, beginTime, endTime);
				}
			}
			
			//write data header
			if (dataStamp != pdi->prevDataStamp || pdi->firstRun || numLostPackets > 0)
			{
				//typical situation - pulse marking by time in nanosecond
				char* format = "\n#time_[ns] %llu\n";
				unsigned long long pulseMark = timeStamp_HWticks * hwPeriod_ns;

				if (measureParams.pulse_mark_by_number)
				{
					format = "\n#pulse_number %llu\n";
					pulseMark = dataStamp;
				}

				if (measureParams.storeChannel1)
				{
					pdi->HDDparams.bufHDDpointer_CH1 += sprintf(pdi->HDDparams.bufHDDpointer_CH1, format, (unsigned long long)(pulseMark));
				}
				if (measureParams.storeChannel2)
				{
					pdi->HDDparams.bufHDDpointer_CH2 += sprintf(pdi->HDDparams.bufHDDpointer_CH2, format, (unsigned long long)(pulseMark));
				}
			}

			//write data
			if (measureParams.storeOnlyOneValue)
			{
				if (measureParams.storeChannel1)
					WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH1), 1);

				if (measureParams.storeChannel2)
					WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH2), 1);
			}
			else
			{
				int i = PACKET_HEADER_LENGTH;
				while (i < data_len - PACKET_FOOTER_LENGTH)
				{
					int sample;
					for (sample = 0; sample < HW_SAMPLES_IN_1_PERIOD && i < data_len; sample+=2, i+=3)
					{
						unsigned short sampleA = GetSampleA(data, i);
						unsigned short sampleB = GetSampleB(data, i);

						if ((mode == ADC_CH1_fast))//measureParams.storeChannel1 && measureParams.ACD12_dualEdgeSamplingMode)
						{
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH1), sampleA);
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH1), sampleB);
						}
						if (mode == ADC_fused)
						{
							WriteUnsignedInt(&(pdi->HDDparams.bufHDDpointer_CH1), (sampleA<<12) | sampleB);
						}
						if (mode == ADC_CH2_fast)//(measureParams.storeChannel2 && measureParams.ACD12_dualEdgeSamplingMode)
						{
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH2), sampleA);
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH2), sampleB);
						}
						if ((mode == ADC_CH1) || (mode == ADC_CH1_2))//(measureParams.storeChannel1 && !measureParams.ACD12_dualEdgeSamplingMode)
						{
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH1), sampleB);
						}
						if ((mode == ADC_CH2) || (mode == ADC_CH1_2))//(measureParams.storeChannel2 && !measureParams.ACD12_dualEdgeSamplingMode)
						{
							WriteUnsignedShort(&(pdi->HDDparams.bufHDDpointer_CH2), sampleA);
						}
					}
					timeStamp_HWticks += 1;
				}
			}
			
			//write num packets stored to HDD
			time_t actualTime;
			actualTime = time ( NULL );
			if (actualTime != previousTime)
			{
				PrintSavedToHDD();
			}
			previousTime = actualTime;
		}
		else
		{
			fprintf(stderr, "Unknown type of data storing.\n");
			return;
		}
	}
	
	
	if (pdi->HDDparams.bufHDDpointer_CH1 >= pdi->HDDparams.bufHDDstop_CH1)
	{
		FlushHDDbuf_CH1(&(pdi->HDDparams));
	}
	if (pdi->HDDparams.bufHDDpointer_CH2 >= pdi->HDDparams.bufHDDstop_CH2)
	{
		FlushHDDbuf_CH2(&(pdi->HDDparams));
	}

	pdi->prevPacketTimeEnd_HWticks = timeStamp_HWticks;
	pdi->prevDataStamp = dataStamp;
	pdi->prevPacketStamp = packetStamp;
	pdi->firstRun = 0;
}

void PrintLostPackets(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2)
{
	printf("Lost packets      | %16llu | \n\n\n", pdi_ADC1->lostPackets + pdi_ADC2->lostPackets);
}

void ReadAndWriteData(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2)
{	
	time_t lastAutoStore = time(NULL);

	//read and write data
	int writeData = 1;
	int autoStop = 0;
	while (runProgram == 1)
	{
		//buffering data from ethernet to buffer
		if (writeData)
		{
			if (measureParams.ADC1_on)
			{
				BufferPacket(pdi_ADC1);
			}
			if (measureParams.ADC2_on)
			{
				BufferPacket(pdi_ADC2);
			}
			
			//compute the autostore while the measuring
			int autoStore = 0;
			if (measureParams.autoStoreSeconds > 0 && (time(NULL) - lastAutoStore) >= measureParams.autoStoreSeconds)
			{
				autoStore = 1;
				lastAutoStore = time(NULL);
			}
			
			//compute autoStop
			if (measureParams.autoStopPulses > 0 && (b.bufferedPackets[0] + b.bufferedPackets[1]) > measureParams.autoStopPulses)
			{
				runProgram = 0;
				autoStore = 1;
				autoStop = 1;
			}

			if (b.bufPointerWrite > b.bufStop || autoStore == 1 || measureParams.output == OUTPUT_UDP)		//if data is transvered via UDP, it is necesseary to decode sze packets one by one
			{
				writeData = 0;

				if (!measureParams.ACD12_testPatternMode && !measureParams.ACD12_timeTestMode)
				{
					if (measureParams.output == OUTPUT_FILE)
					{
						PrintBufferedPackets(1, 0, 1);
						printf("\n");
					}
				}
			}
		}
		//storing data from buffer to HDD or UDP
		else
		{
			if (b.bufPointerRead >= b.bufPointerWrite)
			{
				writeData = 1;
				b.bufPointerWrite = b.buffer;
				b.bufPointerRead = b.buffer;

				if (!measureParams.ACD12_testPatternMode && !measureParams.ACD12_timeTestMode && measureParams.output == OUTPUT_FILE)
				{
					PrintSavedToHDD();
				}

				//write statistics at the end of saving to HDD or sending to UDP
				if (!measureParams.ACD12_testPatternMode && !measureParams.ACD12_timeTestMode)
				{
					if (measureParams.output == OUTPUT_UDP)
					{
						PrintUDPStatistics(pdi_ADC1, pdi_ADC2, 1);
					}
					else if (measureParams.output == OUTPUT_FILE)
					{
						PrintLostPackets(pdi_ADC1, pdi_ADC2);
					}
				}
				
				//flush HDD buffers
				if (measureParams.ADC1_on)
				{
					if (measureParams.storeChannel1)
					{
						FlushHDDbuf_CH1(&(pdi_ADC1->HDDparams));
					}
					if (measureParams.storeChannel2)
					{
						FlushHDDbuf_CH2(&(pdi_ADC1->HDDparams));
					}
				}
				if (measureParams.ADC2_on)
				{
					if (measureParams.storeChannel1)
					{
						FlushHDDbuf_CH1(&(pdi_ADC2->HDDparams));
					}
					if (measureParams.storeChannel2)
					{
						FlushHDDbuf_CH2(&(pdi_ADC2->HDDparams));
					}
				}
			}
			else
			{
				//when no packet incomming while packet reading and DecodeAndStorePacket is called, the last incomming packet is repeatly decoded
				//therefore the DecodeAndStorePacket is in else branch
				DecodeAndStorePacket(pdi_ADC1, pdi_ADC2);
			}
		}
	}
	
	if (writeData)
	{
		PrintBufferedPackets(1, 0, 1);
	}
	
	if (autoStop)
	{
		printf("Auto stop on %u pulses\n", measureParams.autoStopPulses);
	}
}



void SwitchADCsOn()
{
	printf("Starting measurement.\t\t");fflush(stdout);
	if (MeasureStart(&measureParams)) printf("\tOK\n");
	else printf("\tFALSE\n");
	fflush(stdout);
}

void SwitchADCsOff()
{
	//disable ADC output (to stop data transfer throught PCIe)
	printf("Stopping measurement.\t\t");fflush(stdout);
	if (MeasureStop(&measureParams)) printf("\tOK\n");
	else printf("\tFALSE\n");
	fflush(stdout);

}

void OpenOutputFiles(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2, char* filePrefix)
{
	//open output files  
	char outFile_ADC1_CH1_name[50];
	sprintf(outFile_ADC1_CH1_name, measureParams.ACD12_dualEdgeSamplingMode ? "%.40s_ADC1_CH1_fast.dat" : "%.40s_nezes.dat", filePrefix);

	char outFile_ADC1_CH2_name[50];
	sprintf(outFile_ADC1_CH2_name, measureParams.ACD12_dualEdgeSamplingMode ? "%.40s_ADC1_CH2_fast.dat" : "%.40s_zesil.dat", filePrefix);

	char outFile_ADC2_CH1_name[50];
	sprintf(outFile_ADC2_CH1_name, measureParams.ACD12_dualEdgeSamplingMode ? "%.40s_ADC2_CH1_fast.dat" : "%.40s_ADC2_CH1_slow.dat", filePrefix);

	char outFile_ADC2_CH2_name[50];
	sprintf(outFile_ADC2_CH2_name, measureParams.ACD12_dualEdgeSamplingMode ? "%.40s_ADC2_CH2_fast.dat" : "%.40s_ADC2_CH2_slow.dat", filePrefix);

	if (measureParams.ADC1_on)
	{
		if (measureParams.storeChannel1)
		{
			pdi_ADC1->HDDparams.outFile_CH1 = OpenFile(outFile_ADC1_CH1_name);
			WriteFileHeader(pdi_ADC1->HDDparams.outFile_CH1, measureParams.ACD12_dualEdgeSamplingMode ? "ADC1,CH1,fast" : "ADC1,CH1,slow", 
					measureParams.ACD12_dualEdgeSamplingMode, measureParams.ADC1_threshold, measureParams.ADC1_offsetPrev, measureParams.ADC1_offsetPost, measureParams.ADC1_threshold_channel, measureParams.ADC1_CH1_scale, measureParams.ADC1_CH1_offset);
		}
		if (measureParams.storeChannel2)
		{
			pdi_ADC1->HDDparams.outFile_CH2 = OpenFile(outFile_ADC1_CH2_name);
			WriteFileHeader(pdi_ADC1->HDDparams.outFile_CH2, measureParams.ACD12_dualEdgeSamplingMode ? "ADC1,CH2,fast" : "ADC1,CH2,slow", 
					measureParams.ACD12_dualEdgeSamplingMode, measureParams.ADC1_threshold, measureParams.ADC1_offsetPrev, measureParams.ADC1_offsetPost, measureParams.ADC1_threshold_channel, measureParams.ADC1_CH2_scale, measureParams.ADC1_CH2_offset);
		}
	}
	if (measureParams.ADC2_on)
	{
		if (measureParams.storeChannel1)
		{
			pdi_ADC2->HDDparams.outFile_CH1 = OpenFile(outFile_ADC2_CH1_name);
			WriteFileHeader(pdi_ADC2->HDDparams.outFile_CH1, measureParams.ACD12_dualEdgeSamplingMode ? "ADC2,CH1,fast" : "ADC2,CH1,slow", 
					measureParams.ACD12_dualEdgeSamplingMode, measureParams.ADC2_threshold, measureParams.ADC2_offsetPrev, measureParams.ADC2_offsetPost, measureParams.ADC2_threshold_channel, measureParams.ADC2_CH1_scale, measureParams.ADC2_CH1_offset);
		}
		if (measureParams.storeChannel2)
		{	
			pdi_ADC2->HDDparams.outFile_CH2 = OpenFile(outFile_ADC2_CH2_name);
			WriteFileHeader(pdi_ADC2->HDDparams.outFile_CH2, measureParams.ACD12_dualEdgeSamplingMode ? "ADC2,CH2,fast" : "ADC2,CH2,slow", 
					measureParams.ACD12_dualEdgeSamplingMode, measureParams.ADC2_threshold, measureParams.ADC2_offsetPrev, measureParams.ADC2_offsetPost, measureParams.ADC2_threshold_channel, measureParams.ADC2_CH2_scale, measureParams.ADC2_CH2_offset);
		}
	}
}

int PrepareAndStart(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2, unsigned char openFiles, char* filePrefix)
{
	printf("\nPREPARE INTERFEACE\n");

	PacketDecodingInfoInit(pdi_ADC1, ADC1_ID);
	PacketDecodingInfoInit(pdi_ADC2, ADC2_ID);

	unsigned long long bufferSize = 0;
	if (measureParams.ACD12_testPatternMode || measureParams.ACD12_timeTestMode)
	{
		bufferSize = BUFFER_LENGTH_TESTING;
	}
	else
	{
		bufferSize = BUFFER_LENGTH_READING;
	}

	if (bufferInit(&b, bufferSize)==0) return 0;

	if (openFiles)
	{
		OpenOutputFiles(pdi_ADC1, pdi_ADC2, filePrefix);
	}

	SwitchADCsOn();

	runProgram = 1;
	return 1;
}



unsigned short GetMean(unsigned long long N, unsigned long long sum)
{
	return (unsigned short)(sum / N) & 0x0FFF;
}

unsigned short GetStdDeviation(unsigned long long N, unsigned long long sumPow2, unsigned long long mean)
{
	return (unsigned short)sqrt((sumPow2 / N) - (mean*mean)) & 0x0FFF;
}



void CloseFiles(tPacketDecodingInfo *pdi_ADC1, tPacketDecodingInfo *pdi_ADC2)
{
	if (measureParams.ADC1_on)
	{
		if (measureParams.storeChannel1)
		{
			FlushHDDbuf_CH1(&(pdi_ADC1->HDDparams));
			WriteFileHeaderStatistics(pdi_ADC1->HDDparams.outFile_CH1, pdi_ADC1->catchedPackets, pdi_ADC1->lostPackets);
			fclose(pdi_ADC1->HDDparams.outFile_CH1);

		}
		if (measureParams.storeChannel2)
		{
			FlushHDDbuf_CH2(&(pdi_ADC1->HDDparams));
			WriteFileHeaderStatistics(pdi_ADC1->HDDparams.outFile_CH2, pdi_ADC1->catchedPackets, pdi_ADC1->lostPackets);
			fclose(pdi_ADC1->HDDparams.outFile_CH2);

		}
	}
	if (measureParams.ADC2_on)
	{
		if (measureParams.storeChannel1)
		{
			FlushHDDbuf_CH1(&(pdi_ADC2->HDDparams));
			WriteFileHeaderStatistics(pdi_ADC2->HDDparams.outFile_CH1, pdi_ADC2->catchedPackets, pdi_ADC2->lostPackets);
			fclose(pdi_ADC2->HDDparams.outFile_CH1);

		}
		if (measureParams.storeChannel2)
		{
			FlushHDDbuf_CH2(&(pdi_ADC2->HDDparams));
			WriteFileHeaderStatistics(pdi_ADC2->HDDparams.outFile_CH2, pdi_ADC2->catchedPackets, pdi_ADC2->lostPackets);
			fclose(pdi_ADC2->HDDparams.outFile_CH2);

		}
	}

	//free buffers
	free(pdi_ADC1->HDDparams.bufferHDD_CH1);
	free(pdi_ADC1->HDDparams.bufferHDD_CH2);
	free(pdi_ADC2->HDDparams.bufferHDD_CH1);
	free(pdi_ADC2->HDDparams.bufferHDD_CH2);
}


void PrintHelp()
{
	printf("Usage: read <configuration file> <output file prefix>\n");
	printf("       read pttm\n");
	printf("Read data from ADC combo card and save it to file. File format is described in TRAC.\n");
	printf("pttm - print assumpted packets for time test mode\n");
	printf("Press ctrl+c to exit program.\n");
}


void WaitPremplifierOnOff(char* message)
{
	//Crl+C is catched by special function
	signal(SIGINT, signal_handler);		
	
	printf(message);
	fflush(stdout);
	
	//wait to press Ctrl+C
	runProgram = 1;
	while (runProgram == 1)
	{}
	
	//Ctrl+C is handled default
	signal(SIGINT, SIG_DFL);
	runProgram = 1;
}



int main(int argc, char **argv)
{	
	MeasureParamsInit(&measureParams);

	//print packet assumptions
	if (argc == 2 && strcmp(argv[1], "pttm") == 0)
	{
		PrintPacketAssumptions(OFFSET_PREV, OFFSET_POST);
		return 0;
	}

	//write help
	if (argc != 3)
	{
		PrintHelp();
		return 0;
	}
	
	printf("\nSpektromeasure - application to configure and read data from FD-11 SOHAJ\n");

	//config file
	printf("------------------------------------------------------------------------\n");
	printf("READ PARAMETERS FROM CONFIG FILE\n\n");
	char *fileConfigName = argv[1];
	if (!ReadConfigFile(fileConfigName, &measureParams))
	{
		exit(1);
	}
	printf("\n------------------------------------------------------------------------\n");
	printf("ADC AND FPGA SETTING\n");

	printf("\nOPENING INTERFACE\n");fflush(stdout);
	if (OpenInterface(&measureParams)==0) {printf("\tFALSE\n");fflush(stdout);return 0;}
	printf("\tOK\n");fflush(stdout);

	printf("\nCLEARING INTERFACE\n");fflush(stdout);
	MeasureStop(&measureParams);
	if (ClearInterface(&measureParams)==0) {CloseInterface(&measureParams);printf("\tFALSE\n");fflush(stdout);return 0;}
	printf("\tOK\n");fflush(stdout);
	
	if (measureParams.output == OUTPUT_UDP)
	{
		printf("\nPREPARE UDP INTERFEACE\n"); fflush(stdout);
		if (PrepareUDP(measureParams.UDP_IP_address) == 0) 
		{
			CloseInterface(&measureParams);
			printf("\tFALSE\n");
			fflush(stdout);
			return 0;
		}
		printf("\tOK\n");fflush(stdout);
	}

	printf("\nLOAD MEASURING PARAMETERS INTO FPGA\n");fflush(stdout);
	if (ADCs_LoadParams(&measureParams)==0) {CloseInterface(&measureParams);printf("\tFALSE\n");return 0;}
	//wait for a moment for data to stabilize after calibration and params settings
	//otherwise data tend to have saw
	printf("\tOK\n");
		
	//wait the user switch off the preamplifier
	WaitPremplifierOnOff("\nSwitch off the preamplifier and press Ctrl+C.\n");
	
	printf("\nCALIBRATE ADCs\n");fflush(stdout);
	if (ADCs_Calibrate(&measureParams, 1)==0) {CloseInterface(&measureParams);printf("\tFALSE\n");fflush(stdout);return 0;}
	sleep(2);
	if (ADCs_Calibrate(&measureParams, 0)==0) {CloseInterface(&measureParams);printf("\tFALSE\n");fflush(stdout);return 0;}
	printf("\tOK\n");fflush(stdout);
	
	//wait the user switch on the preamplifier
	WaitPremplifierOnOff("\nCalibration is finished. Switch on the preamplifier and press Ctrl+C.\n");	
	
	
	printf("\n------------------------------------------------------------------------\n");

	tPacketDecodingInfo pdi_ADC1;
	tPacketDecodingInfo pdi_ADC2;
	hwPeriod_ns = (1000.0 / (double)HW_CLOCK_MHZ);
	
	//test pattern mode
	if (measureParams.ACD12_testPatternMode)
	{
		printf("TEST PATTERN MODE\n");
		
		if (PrepareAndStart(&pdi_ADC1, &pdi_ADC2, 1, argv[2])==0) return 0;
		
		printf("\n\nChecking test pattern generated in ADC. Press Ctrl+C to exit test.\nTest should run long time (minimum checked packets is 10 000 000).\nNo errors have to arrive.\n\n\n");

		PrintCheckedPackets(0, 0);
	}
	//time test mode
	else if (measureParams.ACD12_timeTestMode)
	{
		printf("TIME TEST MODE\n");
		
		if (PrepareAndStart(&pdi_ADC1, &pdi_ADC2, 1, argv[2])==0) return 0;
		
		PacketAssumptionsPrepare(OFFSET_PREV, OFFSET_POST);

		printf("Checking test packets generated in FPGA. Press Ctrl+C to exit test.\nTest should run long time (minimum checked packets is 10 000 000).\nNo errors have to arrive.\n\n\n");
	}
	//normal measuring
	else 
	{
		
		if (measureParams.output == OUTPUT_UDP) printf("MEASURING AND SEND VIA UDP PACKETS\n");
		else if (measureParams.output == OUTPUT_FILE) printf("MEASURING AND SAVE TO FILE\n");
		else
		{
			fprintf(stderr, "unknown type of output\n");
			return 0;
		}
		
		if (PrepareAndStart(&pdi_ADC1, &pdi_ADC2, 1, argv[2]) == 0) return 0;
		
		if (measureParams.output == OUTPUT_UDP) PrintUDPStatistics(&pdi_ADC1, &pdi_ADC2, 0);
		else if (measureParams.output == OUTPUT_FILE) PrintBufferedPackets(0, 0, 0);
	}

	fflush(stdout);


	//storing data
	//Crl+C is catched by special function
	signal(SIGINT, signal_handler);

	ReadAndWriteData(&pdi_ADC1, &pdi_ADC2);

	printf("\n");
	SwitchADCsOff();
	runProgram = 1;

	if (!measureParams.ACD12_testPatternMode && !measureParams.ACD12_timeTestMode && measureParams.output == OUTPUT_FILE)
	{
		printf("\nSAVE REST OF DATA TO HDD\n");
		printf("Wait for process done or press ctrl+c.\n\n\n");
	

		while (b.bufPointerRead < b.bufPointerWrite && runProgram)
		{
			DecodeAndStorePacket(&pdi_ADC1, &pdi_ADC2);
		}

	
		PrintSavedToHDD();
		PrintLostPackets(&pdi_ADC1, &pdi_ADC2);

		if (!runProgram)
		{
			printf("\n\nProgram was terminated before store all data to HDD.\n");
		}
		
		printf("\n");
	}
	
	

	

	//close szedata and output files and UDP
	CloseInterface(&measureParams);
	CloseFiles(&pdi_ADC1, &pdi_ADC2);
	if (measureParams.output == OUTPUT_UDP)
	{
		CloseUDP();
	}

	//free main buffer
	free(b.buffer);

	//Ctrl+C is handled default
	signal(SIGINT, SIG_DFL);

	return 0;
}	
