/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : Communication with ADC card through ethernet interface and pcap
 *
 *  Author(s) : Martin CECH <256357@mail.muni.cz>               
 *
 *  Date      : $Id: standalone.c 332 2014-03-13 15:12:37Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <pcap.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>
#include <assert.h>
#include <stdlib.h>
#include "standalone.h"


static pcap_t *eth=NULL;
static FILE *bitstream = NULL;

int openPcapDev(char *devName){
	char *dev;
	pcap_if_t *devs=NULL;
	char errbuf[PCAP_ERRBUF_SIZE];
	if (devName == NULL ) {
		//ask pcap to get list of devices
		pcap_if_t *actDev;
		int i, select=0;
		char buf[2];

		int ret = pcap_findalldevs(&devs, errbuf);
		if(ret != 0){
			fprintf(stderr, "%s, perhaps run with root privileges?\n",errbuf);
			return 1;
		}else if (devs==NULL){
			fprintf(stderr, "No devices found! Perhaps run with root privileges?\n",errbuf);
			return 1;
		}

		printf("Found devices:\n");
		for (actDev = devs, i=1; actDev != NULL; actDev = actDev->next, i++)
		{
			printf("\t %d: %s\t%s\n", i, actDev->name, actDev->description);
		}
		i--;
		do{
			printf("Select[1..%d]: \n", i);
			fgets(buf, 2, stdin);
			select = atoi(buf);
		}while(select < 1 || select > i);
		for (actDev = devs, i=1; i!=select; actDev = actDev->next, i++){}  
		dev=actDev->name;
	}else{
		dev = devName;
	}

	//open pcap with maximum size of packet in promisc mode and 0ms timeout
	eth = pcap_open_live(dev, 0xFFFF, 1, 0, errbuf);
	if (eth == NULL){
		fprintf(stderr, "Cannot open device '%s'!\n", dev);
		return 1;
	}	
	if (devs != NULL) pcap_freealldevs(devs);
	return 0;
}

void closePcapDev()
{
	pcap_close(eth);
	if (bitstream != NULL) fclose(bitstream);
	eth=NULL;
}

void sigalrm_handler(int signum)
{
#ifdef DEBUG_PRINT_INFO
	printf("SIGALRM received\n");fflush(stdout);
#endif
	pcap_breakloop(eth);
}

int checkExpectedData(const unsigned char *sndbuf, const unsigned char *rcvbuf, const unsigned int bufLen)
{
	assert((sndbuf!=NULL) && (rcvbuf!=NULL));

	if (bufLen < CONFIG_PACKET_LEN+HEADER_LEN) return 0;
	int i;
	for (i=0;i<HEADER_LEN;i++){

		if (sndbuf[i] != rcvbuf[i]){
#ifdef DEBUG_PRINT_INFO
			printf("checkExpectedData: header wrong\n");
#endif
			return 0;
		}
	}

	if ((sndbuf[HEADER_LEN]|0x80) != rcvbuf[HEADER_LEN]){
#ifdef DEBUG_PRINT_INFO
			printf("checkExpectedData: bad response msg num wrong\n");
#endif
		return 0;
	}

	for (i=HEADER_LEN+1; i < bufLen; i++){
		if (sndbuf[i] != rcvbuf[i]) {
#ifdef DEBUG_PRINT_INFO
			printf("Wrong byte %d [send: %.2x / rcvd: %.2x]\n",i+1, sndbuf[i], rcvbuf[i]);
#endif
			return 0;}
	}
	return 1;
}

const unsigned char *rcvPacket(int *length, int timeoutms, int clear)
{
	//interrupt settings

	sigset_t block_all, block_mask, block_none;
	sigfillset(&block_all);
	sigfillset(&block_mask);
	sigdelset(&block_mask, SIGALRM);
	sigdelset(&block_mask, SIGINT);
	sigemptyset(&block_none);

	struct sigaction sig_service =
	{
		.sa_handler = sigalrm_handler,
		.sa_mask = block_all,
	};
	//allow interrupt
	sigaction(SIGALRM, &sig_service, NULL);
	sigprocmask(SIG_SETMASK, &block_none, NULL);

	//timer settings
	struct itimerval tval={{0,0}, {timeoutms/1000, (timeoutms%1000)*1000}};
	//run timer
	setitimer(ITIMER_REAL, &tval,NULL);

	const unsigned char *ret;

	if (clear) {
		struct pcap_pkthdr *pinfo;
		while(pcap_next_ex(eth, &pinfo, &ret)==1){setitimer(ITIMER_REAL, &tval,NULL);}
	}else {
		ret = readPcapNext(length);
	}
	//stop timer
	tval.it_value.tv_sec = 0;
	tval.it_value.tv_usec = 0;
	setitimer(ITIMER_REAL, &tval,NULL);
	//sigemptyset(&block_none);
	sigaddset(&block_none, SIGALRM);
	sigprocmask(SIG_SETMASK, &block_none, NULL);
	return ret;
}

int sendPacketWaitAck(t_packetType type, tMeasureParams *params, int timeoutms, int resendAttempts)
{
	assert(eth!=NULL);
	unsigned char *packbuf=calloc(type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN, 1);

	packbuf[0]=type;
	unsigned short tmpU16;
	switch (type){
	case PLL_PACK:
		//TODO default values now, later should be set accordingly to config file
	{
		int i,j;
		int reg[6]={0x009F1000, 0x00000002, 0x0327D4E1, 0x00000000, 0x00040887, 0x000B0000};
		for (i=0;i<6;i++)
		{
			for(j=0;j<4;j++)
			{
				packbuf[i*4+j+1] |= (char)(reg[i]>>(24-j*8));
			}
		}
	}
	break;

	case ADC1_PACK:{
		//config field
		packbuf[1] |= (0x80 * params->ADC12_calib)| 0x20;
		switch(params->ADC1_mode){
		case ADC_tpm:
			if (params->ADC12_calib == 0) {packbuf[1] |= 0x10;}
			break;
		case ADC_CH1_fast:
			packbuf[2] |= 0x80; //dual edge sampling
			break;
		case ADC_CH2_fast:
			packbuf[2]  |= 0x80;
			packbuf[2]  |= 0x40; //channel 2
			break;
		default: break;
		}
		//I-offset field
		tmpU16 =params->ADC1_CH1_offset < 0?0x1000:0x0000 |
				transformScaleCHOffset(params->ADC1_CH1_offset<0?params->ADC1_CH1_offset*-1:params->ADC1_CH1_offset);
		packbuf[3] |= (char)(tmpU16>>8);
		packbuf[4] |= (char)tmpU16;
		//I-FSR field
		tmpU16 = transformScaleCHRange(params->ADC1_CH1_scale);
		packbuf[5] |= (char)(tmpU16>>8);
		packbuf[6] |= (char)tmpU16;
		//Q-offset field
		tmpU16 =params->ADC1_CH2_offset < 0?0x1000:0x0000 |
				transformScaleCHOffset(params->ADC1_CH2_offset<0?params->ADC1_CH2_offset*-1:params->ADC1_CH2_offset);
		packbuf[7] |= (char)(tmpU16>>8);
		packbuf[8] |= (char)tmpU16;
		//Q-FSR field
		tmpU16 = transformScaleCHRange(params->ADC1_CH2_scale);
		packbuf[9] |= (char)(tmpU16>>8);
		packbuf[10] |= (char)tmpU16;
		//thres field
		tmpU16 = params->ADC1_mode == ADC_ttm ? 1 : params->ADC1_threshold;
		packbuf[11] |= (char)(tmpU16>>8);
		packbuf[12] |= (char)tmpU16;
		//threshold offest prev
		tmpU16 = params->ADC1_mode == ADC_ttm ? OFFSET_PREV : params->ADC1_offsetPrev;
		packbuf[13] |= (char)(tmpU16>>8);
		packbuf[14] |= (char)tmpU16;
		//threshold offset post
		tmpU16 = params->ADC1_mode == ADC_ttm ? OFFSET_POST : params->ADC1_offsetPost;
		packbuf[15] |= (char)(tmpU16>>8);
		packbuf[16] |= (char)tmpU16;
		//method sel register
		switch (params->ADC1_mode){
		case ADC_CH1_fast:case ADC_CH2_fast:case ADC_tpm:case ADC_ttm:case ADC_CH1_2:case ADC_CH1:case ADC_CH2:
			tmpU16 = 0x0000;
			break;
		case ADC_fused:
			tmpU16 = (1<<15);
			break;
		case ADC_INTM_CH1:
			tmpU16 = (0<<15) | (1 << 11) | params->ADC1_integration_delta;
			break;
		case ADC_INTM_CH2:
			tmpU16 = (0<<15) | (1 << 12) | params->ADC1_integration_delta;
			break;
		case ADC_INTM_fused:
			tmpU16 = (1<<15) | (1 << 12) | params->ADC1_integration_delta;
			break;
		default:break;
		}
		packbuf[17] |= (char)(tmpU16>>8);
		packbuf[18] |= (char)tmpU16;
		
		//method params registr 1
		switch (params->ADC1_mode){
		case ADC_CH1_fast:case ADC_CH2_fast:case ADC_tpm:case ADC_ttm:case ADC_CH1_2:
		case ADC_CH1:case ADC_CH2:case ADC_fused:
			tmpU16 = 0x0000;
			break;
		case ADC_INTM_CH1:case ADC_INTM_CH2:case ADC_INTM_fused:
			tmpU16 = params->ADC1_integration_sub;
			break;
		default:break;
		}
		packbuf[19] |= (char)(tmpU16>>8);
		packbuf[20] |= (char)tmpU16;
		if ((params->ADC1_mode == ADC_INTM_fused) || (params->ADC1_mode == ADC_fused)){
			//method params registr 2
			tmpU16 = params->ADC1_fuser_max;
			packbuf[21] |= (char)(tmpU16>>8);
			packbuf[22] |= (char)tmpU16;
			//method params registr 3
			tmpU16 = transformFuserMult(params->ADC1_fuser_mult);
			packbuf[23] |= (char)(tmpU16>>8);
			packbuf[24] |= (char)tmpU16;
			//method params registr 4
			tmpU16 = params->ADC1_fuser_add;
			packbuf[25] |= (char)(tmpU16>>8);
			packbuf[26] |= (char)tmpU16;
		}
	}
	break;

	case ADC2_PACK:{
		//config field
		packbuf[1] |= (0x80 * params->ADC12_calib)| 0x20;
		switch(params->ADC2_mode){
		case ADC_tpm:
			if (params->ADC12_calib == 0) {packbuf[1] |= 0x10;}
			break;
		case ADC_CH1_fast:
			packbuf[2] |= 0x80; //dual edge sampling
			break;
		case ADC_CH2_fast:
			packbuf[2]  |= 0x80;
			packbuf[2]  |= 0x40; //channel 2
			break;
		default: break;
		}
		//I-offset field
		tmpU16 =params->ADC2_CH1_offset < 0?0x1000:0x0000 |
				transformScaleCHOffset(params->ADC2_CH1_offset<0?params->ADC2_CH1_offset*-1:params->ADC2_CH1_offset);
		packbuf[3] |= (char)(tmpU16>>8);
		packbuf[4] |= (char)tmpU16;
		//I-FSR field
		tmpU16 = transformScaleCHRange(params->ADC2_CH1_scale);
		packbuf[5] |= (char)(tmpU16>>8);
		packbuf[6] |= (char)tmpU16;
		//Q-offset field
		tmpU16 =params->ADC2_CH2_offset < 0?0x1000:0x0000 |
				transformScaleCHOffset(params->ADC2_CH2_offset<0?params->ADC2_CH2_offset*-1:params->ADC2_CH2_offset);
		packbuf[7] |= (char)(tmpU16>>8);
		packbuf[8] |= (char)tmpU16;
		//Q-FSR field
		tmpU16 = transformScaleCHRange(params->ADC2_CH2_scale);
		packbuf[9] |= (char)(tmpU16>>8);
		packbuf[10] |= (char)tmpU16;
		//thres field
		tmpU16 = params->ADC2_mode == ADC_ttm ? 1 : params->ADC2_threshold;
		packbuf[11] |= (char)(tmpU16>>8);
		packbuf[12] |= (char)tmpU16;
		//threshold offest prev
		tmpU16 = params->ADC2_mode == ADC_ttm ? OFFSET_PREV : params->ADC2_offsetPrev;
		packbuf[13] |= (char)(tmpU16>>8);
		packbuf[14] |= (char)tmpU16;
		//threshold offset post
		tmpU16 = params->ADC2_mode == ADC_ttm ? OFFSET_POST : params->ADC2_offsetPost;
		packbuf[15] |= (char)(tmpU16>>8);
		packbuf[16] |= (char)tmpU16;
		//method sel register
		switch (params->ADC2_mode){
		case ADC_CH1_fast:case ADC_CH2_fast:case ADC_tpm:case ADC_ttm:case ADC_CH1_2:case ADC_CH1:case ADC_CH2:
			tmpU16 = 0x0000;
			break;
		case ADC_fused:
			tmpU16 = (1<<15);
			break;
		case ADC_INTM_CH1:
			tmpU16 = (0<<15) | (1 << 11) | params->ADC2_integration_delta;
			break;
		case ADC_INTM_CH2:
			tmpU16 = (0<<15) | (1 << 12) | params->ADC2_integration_delta;
			break;
		case ADC_INTM_fused:
			tmpU16 = (1<<15) | (1 << 12) | params->ADC2_integration_delta;
			break;
		default:break;
		}
		packbuf[17] |= (char)(tmpU16>>8);
		packbuf[18] |= (char)tmpU16;
		//method params registr 1
		switch (params->ADC2_mode){
		case ADC_CH1_fast:case ADC_CH2_fast:case ADC_tpm:case ADC_ttm:case ADC_CH1_2:
		case ADC_CH1:case ADC_CH2:case ADC_fused:
			tmpU16 = 0x0000;
			break;
		case ADC_INTM_CH1:case ADC_INTM_CH2:case ADC_INTM_fused:
			tmpU16 = params->ADC2_integration_sub;
			break;
		default:break;
		}
		packbuf[19] |= (char)(tmpU16>>8);
		packbuf[20] |= (char)tmpU16;
		if ((params->ADC2_mode == ADC_INTM_fused) || (params->ADC2_mode == ADC_fused)){
			//method params registr 2
			tmpU16 = params->ADC2_fuser_max;
			packbuf[21] |= (char)(tmpU16>>8);
			packbuf[22] |= (char)tmpU16;
			//method params registr 3
			tmpU16 = transformFuserMult(params->ADC2_fuser_mult);
			packbuf[23] |= (char)(tmpU16>>8);
			packbuf[24] |= (char)tmpU16;
			//method params registr 4
			tmpU16 = params->ADC2_fuser_add;
			packbuf[25] |= (char)(tmpU16>>8);
			packbuf[26] |= (char)tmpU16;
		}
	}
	break;
	case FPGA_CONF_PACK:
		packbuf[1] |= ((params->ADC1_on) ? 0x00: 0x01);

		if (params->ADC1_mode == ADC_ttm){
			packbuf[1] |= 0x10;
		}
		if (params->ADC2_mode == ADC_ttm){
			packbuf[1] |= 0x20;
		}

		packbuf[2] |= ((params->ADC2_on) ? 0x00: 0x10) | 0x01 * params->ADC12_oen;

		if(params->standalone & 4){
			packbuf[2] |= 0x08;
		}

		break;

	case PING_PACK:
		for(tmpU16 = 1; tmpU16 < CONFIG_PACKET_LEN+1; tmpU16++){
			packbuf[tmpU16] = tmpU16;
		}
		break;
	case BITSTREAM_PACK:
	{
		static unsigned int bsaddr = 0;
		if (bitstream == NULL){
			bitstream = fopen(params->bitfile, "r");
			if (bitstream == NULL){free(packbuf);return -2;}
			bsaddr = 0;
		}
		if (feof(bitstream)){
			fclose(bitstream);
			bitstream = NULL;
			free(packbuf);
			return 3;
		}
		fread(&packbuf[4], 1, BITSTREAM_DATA_LEN, bitstream);
		//int i;
		//for(i=4;i<BITSTREAM_DATA_LEN+4;i++){
		//	packbuf[i]=0x22;
		//	if (i==BITSTREAM_DATA_LEN+3) packbuf[i]=0x33;
		//}
		packbuf[1]=(bsaddr>>16) & 0xFF;
		packbuf[2]=(bsaddr>>8) & 0xFF;
		packbuf[3]=bsaddr & 0xFF;
		bsaddr+= BITSTREAM_DATA_LEN;
	}
	break;
	case RESET_PACK:
		break;
	case FLASH_ERASE_PACK:
		break;
	default:break;
	}

	unsigned char *sendbuf = malloc((type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+HEADER_LEN);
	memcpy(sendbuf+HEADER_LEN, packbuf, (type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN));
	sendbuf[0]= 0x00;sendbuf[1]= 0x13;sendbuf[2]= 0xE8;sendbuf[3]= 0x20;sendbuf[4]= 0x1F;sendbuf[5]= 0xAE;
	sendbuf[6]= 0x00;sendbuf[7]= 0x13;sendbuf[8]= 0xE8;sendbuf[9]= 0x20;sendbuf[10]= 0x1F;sendbuf[11]= 0xAF;
	sendbuf[12]=0x08;sendbuf[13]=0x00;

	int attempts = resendAttempts + 1;
	int retval = 0;
	const char *rcvBuf;
	int skip_packets;
	int timeoutms_in = timeoutms;
	while (attempts > 0){
#ifdef DEBUG_PRINT_INFO
		{
			int i;
			printf("Sending packet: \n");
			for (i = 0; i < HEADER_LEN; i++){
				if ((i!=0)&&(i%2==0)) printf(" %.2X", 0x00FF&(char)sendbuf[i]);
				else printf("%.2X", 0x00FF&(char)sendbuf[i]);
			}
			printf("\n");
			for (i = HEADER_LEN; i < (type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+HEADER_LEN; i++){
				if (i%2==0) printf("%.2X ", 0x00FF&(char)sendbuf[i]);
				else printf("%.2X", 0x00FF&(char)sendbuf[i]);

				if ((i-HEADER_LEN)%16==0) printf("\n");
			}
			printf("\n");
			fflush(stdout);
		}
		printf("cekam na enter\n");
				getchar();
#endif

		if (pcap_sendpacket(eth, sendbuf, (type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+HEADER_LEN)<0){
			free(packbuf);
			free(sendbuf);
			return -1;
		}

		int rcvLen=HEADER_LEN+(type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+1;
		skip_packets = 100000;
		while((rcvLen!=(HEADER_LEN+(type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)))&&(skip_packets>0)){
			rcvBuf = rcvPacket(&rcvLen, timeoutms_in, 0);
			if (rcvBuf == NULL) {
				//no packet avaible -> no response from card
				rcvLen=HEADER_LEN+(type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+1;
				break;
			}else {
				//bad packet len, probably measuring data, skip them and wait for ack packet
				skip_packets--;
			}
		}
#ifdef DEBUG_PRINT_INFO
		{
			int i;
			int k=1;
			if(rcvLen != (HEADER_LEN+(type==BITSTREAM_PACK?BITSTREAM_PACKET_LEN:CONFIG_PACKET_LEN)+1)){
				printf("Received data len: %d\n%d: ", rcvLen, k);
			}else{
				printf("No packet received!\n");
			}
			if (rcvBuf!=NULL && (rcvLen>HEADER_LEN)){
				for (i = 0; i < HEADER_LEN; i++){
					if ((i!=0)&&(i%2==0)) printf(" %.2X", 0x00FF&(char)rcvBuf[i]);
					else printf("%.2X", 0x00FF&(char)rcvBuf[i]);
				}
				printf("\n%d: ",++k);
				for (i = HEADER_LEN; i < rcvLen; i++){
					if (i%2==0) printf("%.2X ", 0x00FF&(char)rcvBuf[i]);
					else printf("%.2X", 0x00FF&(char)rcvBuf[i]);

					if ((i-HEADER_LEN)%16==0) printf("\n%d: ",++k);
				}
				fflush(stdout);
			}
			printf("\n-----------------------------------------\n");
		}
#endif

		if (rcvBuf == NULL){
			usleep(10000);
			timeoutms_in += 100;
			attempts--;
			retval=0;
		}else if (checkExpectedData(sendbuf, rcvBuf, rcvLen)==0){
			usleep(10000);
			attempts--;
			retval=1;
		}
		else{
			free(packbuf);
			free(sendbuf);
			return 2;
		}
	}
	free(packbuf);
	free(sendbuf);
	return retval;
}

unsigned char *readPcapNext(int *length)
{
	assert(eth!=NULL);
	struct pcap_pkthdr pinfo;
	unsigned char *ret;

	ret= (unsigned char *)pcap_next(eth, &pinfo);
	*length = pinfo.caplen;
	return ret;
}

void clearPcapBuffer(){
	int len;
	rcvPacket(&len, 1, 1);
}
