/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC Data Acquisition to File
 *               
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>             
 *
 *  Date      : $Id: storingData.c 326 2014-03-11 10:35:03Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "storingData.h"

extern double hwPeriod_ns;	//is computed in main()
static char smallBuf[7];

void HDDParamsInit(tHDDparams *p)
{
	p->outFile_CH1 = NULL;
	p->outFile_CH2 = NULL;
	
	p->bufferHDD_CH1 = p->bufHDDpointer_CH1 = malloc(HDD_BUFFER_LENGTH);
	p->bufferHDD_CH2 = p->bufHDDpointer_CH2 = malloc(HDD_BUFFER_LENGTH);
	
	if (p->bufferHDD_CH1 == NULL || p->bufferHDD_CH2 == NULL)
	{
		fprintf(stderr, "Can not allocate HDD buffer.\n");
		abort();
	}
	p->bufHDDstop_CH1 = p->bufferHDD_CH1 + HDD_BUFFER_LENGTH - HDD_BUFFER_STOP;
	p->bufHDDstop_CH2 = p->bufferHDD_CH2 + HDD_BUFFER_LENGTH - HDD_BUFFER_STOP;
	
}
 
 
FILE* OpenFile(char* fileName)
{
	FILE *file = fopen(fileName, "w");
	//FILE *file = fopen("/dev/null", "w");
	if (file == NULL)
	{
		fprintf(stderr, "Can not open output file %s\n", fileName);
		abort();
	}
	
	setbuf(file, NULL);
	return file;
}

void WriteFileHeader(FILE* file, char *comment, int ACD12_dualEdgeSamplingMode, unsigned short threshold, unsigned short offsetPrev, unsigned short offsetPost, char *thresholdOnChannel, float scale, float offset)
{
	//statistics
	fprintf(file, "#catched_packets                                            \n");
	fprintf(file, "#lost_packets                                               \n");

	fprintf(file, "\n");
	
	//measure settings
	fprintf(file, "#data_source                                 %15s\n", comment);
	fprintf(file, "#period_[ns]                                 %15.5f\n", (hwPeriod_ns / (float)HW_SAMPLES_IN_1_PERIOD) * (ACD12_dualEdgeSamplingMode ? 1 : 2));
	fprintf(file, "#threshold                                   %15d\n", threshold & 0x0FFF);
	fprintf(file, "#threshold_on_channel                        %15s\n", thresholdOnChannel);
	fprintf(file, "#offset_prev                                 %15d\n", offsetPrev * HW_SAMPLES_IN_1_PERIOD);
	fprintf(file, "#offset_post                                 %15d\n", offsetPost * HW_SAMPLES_IN_1_PERIOD);
	fprintf(file, "#scale_[mV]                                  %15.2f\n", scale);
	fprintf(file, "#offset_[mV]                                 %15.2f\n", offset);

	fprintf(file, "\n");

	//time and date when measure start
	time_t rawtime;
	struct tm * timeinfo;
	rawtime = time ( NULL );
	timeinfo = localtime (&rawtime);
	char timeToWrite[9];
	char dateToWrite[11];
	strftime(timeToWrite, 9, "%H:%M:%S", timeinfo);
	strftime(dateToWrite, 11, "%Y-%m-%d", timeinfo);
	fprintf(file, "#date_[YYYY_MM_DD]                           %15s\n", dateToWrite );
	fprintf(file, "#time_[HH:MM:SS]                             %15s\n", timeToWrite ); 
	
	fprintf(file, "\n\n");
}

void WriteFileHeaderStatistics(FILE *file, unsigned long long catchedPackets, unsigned long long lostPackets)
{
	//catched_packets      
	fseek(file, 0*LINE_LENGTH + DATA_START, SEEK_SET);
	fprintf(file, "%15llu", catchedPackets);
	
	//lost_packets      
	fseek(file, 1*LINE_LENGTH + DATA_START, SEEK_SET);
	fprintf(file, "%15llu", lostPackets);
}


void inline FlushHDDbuf_CH1(tHDDparams *p)
{
	fwrite(p->bufferHDD_CH1, p->bufHDDpointer_CH1 - p->bufferHDD_CH1, 1, p->outFile_CH1);
	p->bufHDDpointer_CH1 = p->bufferHDD_CH1;
	fflush(p->outFile_CH1);
}

void inline FlushHDDbuf_CH2(tHDDparams *p)
{
	fwrite(p->bufferHDD_CH2, p->bufHDDpointer_CH2 - p->bufferHDD_CH2, 1, p->outFile_CH2);
	p->bufHDDpointer_CH2 = p->bufferHDD_CH2;
	fflush(p->outFile_CH2);
}

void inline WriteHexaValue(char* buffer, unsigned char value)
{
	if (value <= 9)
	{
		buffer[0] = '0' + value;
	}
	else
	{
		buffer[0] = 'A' - 10 + value;
	}
}

void WriteDecimal(char **buffer, unsigned long value)
{
	*buffer += sprintf(*buffer, " %lu", value);
}

void WriteUnsignedInt(char** buffer, unsigned int value)
{
	unsigned char nums[6] ={
			(unsigned char)(value >> 5*4) & 0xF,
			(unsigned char)(value >> 4*4) & 0xF,
			(unsigned char)(value >> 3*4) & 0xF,
			(unsigned char)(value >> 2*4) & 0xF,
			(unsigned char)(value >> 1*4) & 0xF,
			(unsigned char)value & 0xF};
	int i,j;
	for (i=0; i<5; i++)
	{
		if (nums[i] != 0) {
			break;
		}
	}
	for (j=0,i;i<6;i++,j++){
		WriteHexaValue(&smallBuf[j], nums[i]);
	}

	smallBuf[j] = '\n';
	j++;
	memcpy(*buffer, smallBuf, j);
	*buffer += j;
}

//max number to write is 4095
void WriteUnsignedShort(char** buffer, unsigned short value)
{
	unsigned char num2 = (unsigned char)value & 0xF;
	unsigned char num1 = (unsigned char)(value >> 1*4) & 0xF;
	unsigned char num0 = (unsigned char)(value >> 2*4) & 0xF;
	
	if (num0 > 0)
	{
		WriteHexaValue(&smallBuf[0], num0);
		WriteHexaValue(&smallBuf[1], num1);
		WriteHexaValue(&smallBuf[2], num2);
		smallBuf[3] = '\n';
		memcpy(*buffer, smallBuf, 4);
		*buffer += 4;
	}
	else
	{
		if (num1 > 0)
		{
			WriteHexaValue(&smallBuf[0], num1);
			WriteHexaValue(&smallBuf[1], num2);
			smallBuf[2] = '\n';
			memcpy(*buffer, smallBuf, 3);
			*buffer += 3;
		}
		else
		{
			WriteHexaValue(&smallBuf[0], num2);
			smallBuf[1] = '\n';
			memcpy(*buffer, smallBuf, 2);
			*buffer += 2;
		}
		
	}
}


void inline WritePacketsLost(char** bufPointer, unsigned int num, unsigned long long begin, unsigned long long end)
{
	*bufPointer += sprintf(*bufPointer, "\n#lost_packets_max %u\n", num);
	*bufPointer += sprintf(*bufPointer, "#begin %llu\n", begin);
	*bufPointer += sprintf(*bufPointer, "#end %llu\n", end);    
}
